@@ -2,6 +2,7 @@
  * Copyright (C) 2008 Mirco "MacSlow" Müller <macslow@bangang.de>
  * Copyright (C) 2008 daniel g. siegel <dgsiegel@gmail.com>
  * Copyright (C) 2008 Patryk Zawadzki <patrys@pld-linux.org>
+ * Copyright (C) 2008 Andrea Cimitan <andrea.cimitan@gmail.com>
  *
  * Licensed under the GNU General Public License Version 2
  *
@@ -24,17 +25,14 @@
 #endif
 
 #include <stdlib.h>
+#include <glib/gi18n.h>
 #include <gtk/gtk.h>
 #include <librsvg/rsvg.h>
 #include <librsvg/rsvg-cairo.h>
+#include <math.h>
 
 #include "cheese-countdown.h"
 
-#define R 0
-#define G 1
-#define B 2
-#define A 3
-
 #define CHEESE_COUNTDOWN_GET_PRIVATE(o) \
   (G_TYPE_INSTANCE_GET_PRIVATE ((o), CHEESE_TYPE_COUNTDOWN, CheeseCountdownPrivate))
 
@@ -46,6 +44,14 @@ G_DEFINE_TYPE (CheeseCountdown, cheese_countdown, GTK_TYPE_DRAWING_AREA);
 #define STATE_1     3
 #define STATE_SMILE 4
 
+typedef struct
+{
+  gdouble r;
+  gdouble g;
+  gdouble b;
+  gdouble a;
+} CairoColor;
+
 typedef struct
 {
   gint iState;
@@ -53,8 +59,209 @@ typedef struct
   cheese_countdown_cb_t picture_callback;
   cheese_countdown_cb_t hide_callback;
   gpointer callback_data;
+  CairoColor bg;
+  CairoColor text;
 } CheeseCountdownPrivate;
 
+/* Converts from RGB TO HLS */
+static void
+rgb_to_hls (gdouble *r,
+            gdouble *g,
+            gdouble *b)
+{
+  gdouble min;
+  gdouble max;
+  gdouble red;
+  gdouble green;
+  gdouble blue;
+  gdouble h, l, s;
+  gdouble delta;
+
+  red = *r;
+  green = *g;
+  blue = *b;
+
+  if (red > green)
+  {
+    if (red > blue)
+      max = red;
+    else
+      max = blue;
+
+    if (green < blue)
+      min = green;
+    else
+      min = blue;
+  }
+  else
+  {
+    if (green > blue)
+      max = green;
+    else
+      max = blue;
+
+    if (red < blue)
+      min = red;
+    else
+      min = blue;
+  }
+
+  l = (max + min) / 2;
+  if (fabs(max - min) < 0.0001)
+  {
+    h = 0;
+    s = 0;
+  }
+  else
+  {
+    if (l <= 0.5)
+      s = (max - min) / (max + min);
+    else
+      s = (max - min) / (2 - max - min);
+
+    delta = max -min;
+    if (red == max)
+      h = (green - blue) / delta;
+    else if (green == max)
+      h = 2 + (blue - red) / delta;
+    else if (blue == max)
+      h = 4 + (red - green) / delta;
+
+    h *= 60;
+    if (h < 0.0)
+      h += 360;
+  }
+
+  *r = h;
+  *g = l;
+  *b = s;
+}
+
+/* Converts from HLS to RGB */
+static void
+hls_to_rgb (gdouble *h,
+            gdouble *l,
+            gdouble *s)
+{
+  gdouble hue;
+  gdouble lightness;
+  gdouble saturation;
+  gdouble m1, m2;
+  gdouble r, g, b;
+
+  lightness = *l;
+  saturation = *s;
+
+  if (lightness <= 0.5)
+    m2 = lightness * (1 + saturation);
+  else
+    m2 = lightness + saturation - lightness * saturation;
+
+  m1 = 2 * lightness - m2;
+
+  if (saturation == 0)
+  {
+    *h = lightness;
+    *l = lightness;
+    *s = lightness;
+  }
+  else
+  {
+    hue = *h + 120;
+    while (hue > 360)
+      hue -= 360;
+    while (hue < 0)
+      hue += 360;
+
+    if (hue < 60)
+      r = m1 + (m2 - m1) * hue / 60;
+    else if (hue < 180)
+      r = m2;
+    else if (hue < 240)
+      r = m1 + (m2 - m1) * (240 - hue) / 60;
+    else
+      r = m1;
+
+    hue = *h;
+    while (hue > 360)
+      hue -= 360;
+    while (hue < 0)
+      hue += 360;
+
+    if (hue < 60)
+      g = m1 + (m2 - m1) * hue / 60;
+    else if (hue < 180)
+      g = m2;
+    else if (hue < 240)
+      g = m1 + (m2 - m1) * (240 - hue) / 60;
+    else
+      g = m1;
+
+    hue = *h - 120;
+    while (hue > 360)
+      hue -= 360;
+    while (hue < 0)
+      hue += 360;
+
+    if (hue < 60)
+      b = m1 + (m2 - m1) * hue / 60;
+    else if (hue < 180)
+      b = m2;
+    else if (hue < 240)
+      b = m1 + (m2 - m1) * (240 - hue) / 60;
+    else
+      b = m1;
+
+    *h = r;
+    *l = g;
+    *s = b;
+  }
+}
+
+/* Performs a color shading operation */
+static void
+color_shade (const CairoColor * a, float k, CairoColor * b)
+{
+  double red;
+  double green;
+  double blue;
+  double alpha;
+
+  red   = a->r;
+  green = a->g;
+  blue  = a->b;
+  alpha = a->a;
+
+  if (k == 1.0)
+  {
+    b->r = red;
+    b->g = green;
+    b->b = blue;
+    return;
+  }
+
+  rgb_to_hls (&red, &green, &blue);
+
+  green *= k;
+  if (green > 1.0)
+    green = 1.0;
+  else if (green < 0.0)
+    green = 0.0;
+
+  blue *= k;
+  if (blue > 1.0)
+    blue = 1.0;
+  else if (blue < 0.0)
+    blue = 0.0;
+
+  hls_to_rgb (&red, &green, &blue);
+
+  b->r = red;
+  b->g = green;
+  b->b = blue;
+  b->a = alpha;
+}
+
 static gint
 do_text (cairo_t*    pContext,
          gchar*      pcText,
@@ -126,11 +333,19 @@ do_text (cairo_t*    pContext,
 static gboolean
 on_expose (GtkWidget* widget, GdkEventExpose* pEvent, gpointer data)
 {
-  CheeseCountdownPrivate* priv = CHEESE_COUNTDOWN_GET_PRIVATE (widget);
-  cairo_t*  pContext = NULL;
+  CheeseCountdownPrivate *priv = CHEESE_COUNTDOWN_GET_PRIVATE (widget);
+  cairo_t  *pContext = NULL;
+  cairo_pattern_t *pattern;
+  CairoColor bgBorder;
+  CairoColor bgHighlight;
+  CairoColor bgShade1;
+  CairoColor bgShade2;
+  CairoColor bgShade3;
+  CairoColor bgShade4;
   gdouble   fWidth   = (gdouble) widget->allocation.width;
   gdouble   fHeight  = (gdouble) widget->allocation.height;
-  gint      iOffsetX = (widget->allocation.width - 4 * 24) / 2;
+  /* 3 * 26 are the three numbers, 30 is the width of camera-icon.svg */
+  gint      iOffsetX = (widget->allocation.width - 3 * 26 - 30) / 2;
   gint      iOffsetY = (widget->allocation.height - 30) / 2;
   gdouble   fAlpha1;
   gdouble   fAlpha2;
@@ -155,8 +370,19 @@ on_expose (GtkWidget* widget, GdkEventExpose* pEvent, gpointer data)
   else
     fAlphaClick = 0.5f;
 
-  /* clear drawing-context */
+  /* shade the colors used in the pattern */
+  color_shade (&priv->bg, 0.65f, &bgBorder);
+  color_shade (&priv->bg, 1.26f, &bgHighlight);
+  color_shade (&priv->bg, 1.16f, &bgShade1);
+  color_shade (&priv->bg, 1.08f, &bgShade2);
+  color_shade (&priv->bg, 1.00f, &bgShade3);
+  color_shade (&priv->bg, 1.08f, &bgShade4);
+
+  /* create cairo context */
   pContext = gdk_cairo_create (widget->window);
+  cairo_set_line_width (pContext, 1.0f);
+
+  /* clear drawing-context */
   cairo_set_operator (pContext, CAIRO_OPERATOR_CLEAR);
   cairo_paint (pContext);
   cairo_set_operator (pContext, CAIRO_OPERATOR_OVER);
@@ -165,53 +391,89 @@ on_expose (GtkWidget* widget, GdkEventExpose* pEvent, gpointer data)
   cairo_rectangle (pContext, 0.0f, 0.0f, fWidth, fHeight);
   cairo_clip (pContext);
 
-  /* draw the red-ish background */
-  /* glossy version */
-  /*
-  cairo_set_source_rgba (pContext, 1.0f, 0.5f, 0.5f, 1.0f);
-  cairo_rectangle (pContext, 0.0f, 0.0f, fWidth, fHeight);
-  cairo_fill (pContext);
-  cairo_set_source_rgba (pContext, 1.0f, 0.25f, 0.25f, 1.0f);
-  cairo_rectangle (pContext, 0.0f, fHeight / 2.0f, fWidth, fHeight / 2.0f);
-  cairo_fill (pContext);
-  */
-  /* plain version */
-  cairo_set_source_rgba (pContext, 1.0f, 0.25f, 0.25f, 1.0f);
+  /* draw the color-ish background */
   cairo_rectangle (pContext, 0.0f, 0.0f, fWidth, fHeight);
+  pattern = cairo_pattern_create_linear (0.0f, 0.0f, 0.0f, fHeight);
+  cairo_pattern_add_color_stop_rgba (pattern, 0.00f, bgShade1.r, bgShade1.g, bgShade1.b, 1.0f);
+  cairo_pattern_add_color_stop_rgba (pattern, 0.49f, bgShade2.r, bgShade2.g, bgShade2.b, 1.0f);
+  cairo_pattern_add_color_stop_rgba (pattern, 0.49f, bgShade3.r, bgShade3.g, bgShade3.b, 1.0f);
+  cairo_pattern_add_color_stop_rgba (pattern, 1.00f, bgShade4.r, bgShade4.g, bgShade4.b, 1.0f);
+  cairo_set_source (pContext, pattern);
+  cairo_pattern_destroy (pattern);
   cairo_fill (pContext);
 
+  /* draw border */
+  cairo_rectangle (pContext, 0.5f, 0.5f, fWidth-1.0f, fHeight-1.0f);
+  cairo_set_source_rgba (pContext, bgBorder.r, bgBorder.g, bgBorder.b, 0.6f);
+  cairo_stroke (pContext);
+
+  /* draw inner highlight */
+  cairo_rectangle (pContext, 1.5f, 1.5f, fWidth-3.0f, fHeight-3.0f);
+  cairo_set_source_rgba (pContext, bgHighlight.r, bgHighlight.g, bgHighlight.b, 0.5f);
+  cairo_stroke (pContext);
+
+  /* plain version */
+  /*
+  cairo_set_source_rgba (pContext, bg.r, bg.g, bg.b, 1.0f);
+  cairo_rectangle (pContext, 0.0f, 0.0f, fWidth, fHeight);
+  cairo_fill (pContext);
+  */
+
+  char *number;
   /* draw the 3 */
-  cairo_set_source_rgba (pContext, 1.0f, 1.0f, 1.0f, fAlpha3);
+  cairo_set_source_rgba (pContext, priv->text.r, priv->text.g, priv->text.b, fAlpha3);
   cairo_move_to (pContext, (gdouble) iOffsetX, (gdouble) iOffsetY);
+  /* TRANSLATORS:
+   * This is the countdown number when taking the photo.
+   * If you leave as is (that is, %d), it will show 3, 2, 1, 0.
+   * To enable to show the numbers in your own language, use %Id instead.
+   * Please leave the additional whitespace after the number
+   */
+  number = g_strdup_printf (_("%d "), 3);
   iOffsetX += do_text (pContext,
-                       "3 ",
-                       24 * PANGO_SCALE,
+                       number,
+                       26 * PANGO_SCALE,
                        "Bitstream Charter",
                        PANGO_WEIGHT_BOLD,
                        PANGO_STYLE_NORMAL);
   cairo_fill (pContext);
 
   /* draw the 2 */
-  cairo_set_source_rgba (pContext, 1.0f, 1.0f, 1.0f, fAlpha2);
+  cairo_set_source_rgba (pContext, priv->text.r, priv->text.g, priv->text.b, fAlpha2);
   cairo_move_to (pContext, (gdouble) iOffsetX, (gdouble) iOffsetY);
+  /* TRANSLATORS:
+   * This is the countdown number when taking the photo.
+   * If you leave as is (that is, %d), it will show 3, 2, 1, 0.
+   * To enable to show the numbers in your own language, use %Id instead.
+   * Please leave the additional whitespace after the number
+   */
+  number = g_strdup_printf (_("%d "), 2);
   iOffsetX += do_text (pContext,
-                       "2 ",
-                       24 * PANGO_SCALE,
+                       number,
+                       26 * PANGO_SCALE,
                        "Bitstream Charter",
                        PANGO_WEIGHT_BOLD,
                        PANGO_STYLE_NORMAL);
   cairo_fill (pContext);
 
   /* draw the 1 */
-  cairo_set_source_rgba (pContext, 1.0f, 1.0f, 1.0f, fAlpha1);
+  cairo_set_source_rgba (pContext, priv->text.r, priv->text.g, priv->text.b, fAlpha1);
   cairo_move_to (pContext, (gdouble) iOffsetX, (gdouble) iOffsetY);
+  /* TRANSLATORS:
+   * This is the countdown number when taking the photo.
+   * If you leave as is (that is, %d), it will show 3, 2, 1, 0.
+   * To enable to show the numbers in your own language, use %Id instead.
+   * Please leave the additional whitespace after the number
+   */
+  number = g_strdup_printf (_("%d "), 1);
   iOffsetX += do_text (pContext,
-                       "1 ",
-                       24 * PANGO_SCALE,
+                       number,
+                       26 * PANGO_SCALE,
                        "Bitstream Charter",
                        PANGO_WEIGHT_BOLD,
                        PANGO_STYLE_NORMAL);
   cairo_fill (pContext);
+  g_free(number);
 
   /* draw the camera */
   cairo_set_source_surface (pContext,
@@ -235,7 +497,7 @@ redraw_handler (gpointer data)
 }
 
 static cairo_surface_t*
-create_surface_from_svg (gchar* pcFilename)
+create_surface_from_svg (GtkWidget *widget, gchar* pcFilename)
 {
   cairo_surface_t*  pSurface   = NULL;
   RsvgHandle*       pSvgHandle = NULL;
@@ -243,6 +505,8 @@ create_surface_from_svg (gchar* pcFilename)
   cairo_t*          pContext   = NULL;
   RsvgDimensionData dimension;
 
+  CheeseCountdownPrivate* priv = CHEESE_COUNTDOWN_GET_PRIVATE (widget);
+
   rsvg_init ();
 
   /* load svg-file from disk */
@@ -250,7 +514,7 @@ create_surface_from_svg (gchar* pcFilename)
   if (!pSvgHandle)
   {
     g_print ("Could not load file %s!\n", pcFilename);
-    g_error_free(pError);
+    g_error_free (pError);
     return NULL;
   }
 
@@ -259,8 +523,8 @@ create_surface_from_svg (gchar* pcFilename)
 
   /* create a surface-buffer */
   pSurface = cairo_image_surface_create (CAIRO_FORMAT_ARGB32,
-                 dimension.width,
-                 dimension.height);
+                                         dimension.width,
+                                         dimension.height);
   if (cairo_surface_status (pSurface) != CAIRO_STATUS_SUCCESS)
   {
     g_print ("Could not create surface for svg-icon!\n");
@@ -276,8 +540,8 @@ create_surface_from_svg (gchar* pcFilename)
   }
 
   /* clear that context */
-  cairo_set_source_rgba (pContext, 1.0f, 1.0f, 1.0f, 0.0f);
-  cairo_set_operator (pContext, CAIRO_OPERATOR_OVER);
+  cairo_set_source_rgba (pContext, priv->text.r, priv->text.g, priv->text.b, 1.0f);
+  cairo_set_operator (pContext, CAIRO_OPERATOR_XOR);
   cairo_paint (pContext);
 
   /* draw icon into surface-buffer */
@@ -291,6 +555,27 @@ create_surface_from_svg (gchar* pcFilename)
   return pSurface;
 }
 
+static void
+on_style_set_cb (GtkWidget *widget, GtkStyle *previous_style, gpointer data)
+{
+  CheeseCountdownPrivate* priv = CHEESE_COUNTDOWN_GET_PRIVATE (data);
+
+  GdkColor *color_bg = &GTK_WIDGET(widget)->style->bg[GTK_STATE_SELECTED];
+  GdkColor *color_text = &GTK_WIDGET(widget)->style->fg[GTK_STATE_SELECTED];
+  priv->bg.r = ((double)color_bg->red) / 65535;
+  priv->bg.g = ((double)color_bg->green) / 65535;
+  priv->bg.b = ((double)color_bg->blue) / 65535;
+  priv->bg.a = 1.0f;
+  priv->text.r = ((double)color_text->red) / 65535;
+  priv->text.g = ((double)color_text->green) / 65535;
+  priv->text.b = ((double)color_text->blue) / 65535;
+  priv->text.a = 1.0f;
+
+  /* create/load svg-icon */
+  g_free(priv->pSurface);
+  priv->pSurface = create_surface_from_svg (widget, PACKAGE_DATADIR "/pixmaps/camera-icon.svg");
+}
+
 static gboolean
 cheese_countdown_cb (gpointer countdown)
 {
@@ -366,14 +651,14 @@ cheese_countdown_init (CheeseCountdown *countdown)
 {
   CheeseCountdownPrivate* priv = CHEESE_COUNTDOWN_GET_PRIVATE (countdown);
 
-  /* create/load svg-icon */
-  priv->pSurface = create_surface_from_svg (PACKAGE_DATADIR "/pixmaps/camera-icon.svg");
   priv->iState = STATE_OFF;
   priv->picture_callback = NULL;
   priv->hide_callback = NULL;
 
   g_signal_connect (G_OBJECT (countdown), "expose-event",
-        G_CALLBACK (on_expose), NULL);
+                    G_CALLBACK (on_expose), NULL);
+  g_signal_connect (G_OBJECT (countdown), "style-set",
+                    G_CALLBACK (on_style_set_cb), countdown);
 
   g_timeout_add (100, redraw_handler, (gpointer) countdown);
 }
